@@ -141,13 +141,16 @@
 #'   \link[=sampleOrder]{Ordering samples by module summary}
 #' 
 #' @rdname networkProperties
-#' @import ref
 #' @export
 networkProperties <- function(
   network, data, correlation, moduleAssignments=NULL, modules=NULL,
   backgroundLabel="0", discovery=NULL, test=NULL, simplify=TRUE, 
   verbose=TRUE
 ) {
+  # always garbage collect before the function exits so any loaded 
+  # disk.matrices get unloaded as appropriate
+  on.exit({ gc() }, add = TRUE) 
+  
   #----------------------------------------------------------------------------
   # Input processing and sanity checking
   #----------------------------------------------------------------------------
@@ -157,33 +160,41 @@ networkProperties <- function(
   finput <- processInput(discovery, test, network, correlation, data, 
                          moduleAssignments, modules, backgroundLabel,
                          verbose, "props")
-  # Get the loaded datasets
-  dataLoaded <- finput$dataLoaded
-  networkLoaded <- finput$networkLoaded
-  # remove from the finput list so that when we re-assign a new dataset the
-  # memory is freed.
-  finput$dataLoaded <- NULL
-  finput$correlationLoaded <- NULL
-  finput$networkLoaded <- NULL
+  data <- finput$data
+  correlation <- finput$correlation
+  network <- finput$network
+  loadedIdx <- finput$loadedIdx
+  dataEnv <- finput$dataEnv
+  networkEnv <- finput$networkEnv
+  discovery <- finput$discovery
+  test <- finput$test
+  modules <- finput$modules
+  moduleAssignments <- finput$moduleAssignments
+  nDatasets <- finput$nDatasets
+  datasetNames <- finput$datasetNames
   
+  # We don't want a second copy of these environments when we start 
+  # swapping datasets.
+  finput$dataEnv <- NULL
+  finput$networkEnv <- NULL
+  finput$correlationEnv <- NULL # unload the correlation matrix as well since we don't need it
   
   vCat(verbose, 0, "User input ok!")
-  anyDM <- with(finput, {
-    any.disk.matrix(data[[loadedIdx]], correlation[[loadedIdx]], 
-                    network[[loadedIdx]])
-  })
+  anyDM <- any.disk.matrix(data[[loadedIdx]], 
+                           correlation[[loadedIdx]], 
+                           network[[loadedIdx]])
   on.exit({
     vCat(verbose && anyDM, 0, "Unloading dataset from RAM...")
-    rm(dataLoaded, networkLoaded)
+    dataEnv$matrix <- NULL
+    networkEnv$matrix <- NULL
     gc()
   }, add=TRUE)
   
   # Calculate the network properties
-  res <- with(finput, {
-    netPropsInternal(network, data, moduleAssignments, modules, discovery, 
-                     test, nDatasets, datasetNames, verbose, loadedIdx, 
-                     as.ref(dataLoaded), as.ref(networkLoaded), FALSE)
-  })
+  res <- netPropsInternal(network, data, moduleAssignments, 
+                          modules, discovery, test,
+                          nDatasets, datasetNames, verbose,
+                          loadedIdx, dataEnv, networkEnv, FALSE)
   anyDM <- FALSE
   
   # Simplify the output data structure where possible
@@ -216,8 +227,8 @@ networkProperties <- function(
 ###   \code{'processInput'}.
 ### @param verbose logical; should progress be reported? Default is \code{TRUE}.
 ### @param loadedIdx index of the currently loaded dataset.
-### @param dataLoaded reference to currently loaded data matrix (may be NULL).
-### @param networkLoaded reference to currently loaded network matrix.
+### @param dataEnv environment containing the currently loaded data matrix (may be NULL).
+### @param networkEnv environment containing the currently loaded network matrix.
 ### @param keepLast logical; should the dataset processed last be kept in RAM?
 ###   
 ### @return
@@ -227,7 +238,7 @@ networkProperties <- function(
 ### @keywords internal
 netPropsInternal <- function(
   network, data, moduleAssignments, modules, discovery, test, nDatasets, 
-  datasetNames, verbose, loadedIdx, dataLoaded, networkLoaded, keepLast=FALSE
+  datasetNames, verbose, loadedIdx, dataEnv, networkEnv, keepLast=FALSE
 ) {
   # The following declarations are for iterators declared inside each foreach 
   # loop. Declarations are required to satisfy NOTES generated by R CMD check, 
@@ -262,16 +273,16 @@ netPropsInternal <- function(
       # unload previous dataset from RAM
       anyDM <- any.disk.matrix(data[[loadedIdx]], network[[loadedIdx]])
       vCat(verbose && anyDM, 0, "Unloading dataset from RAM...")
-      deref(dataLoaded) <- NULL 
-      deref(networkLoaded) <- NULL
+      dataEnv$matrix <- NULL 
+      networkEnv$matrix <- NULL
       gc()
       
       # Load matrices into RAM if they are 'disk.matrix' objects.
       anyDM <- any.disk.matrix(data[[ti]], network[[ti]])
       vCat(verbose && anyDM, 0, 'Loading matrices of dataset "',
            datasetNames[ti], '" into RAM...', sep="")
-      deref(dataLoaded) <- loadIntoRAM(data[[ti]])
-      deref(networkLoaded) <- loadIntoRAM(network[[ti]])
+      dataEnv$matrix <- loadIntoRAM(data[[ti]])
+      networkEnv$matrix <- loadIntoRAM(network[[ti]])
       loadedIdx <- ti
     }
     
@@ -282,11 +293,11 @@ netPropsInternal <- function(
             datasetNames[ti], '"...', sep="")
         if (is.null(data[[ti]])) {
           props <- NetPropsNoData(
-            deref(networkLoaded), moduleAssignments[[di]], modules[[di]]
+            networkEnv$matrix, moduleAssignments[[di]], modules[[di]]
           )
         } else {
           props <- NetProps(
-            deref(dataLoaded), deref(networkLoaded), moduleAssignments[[di]], 
+            dataEnv$matrix, networkEnv$matrix, moduleAssignments[[di]], 
             modules[[di]]
           )
         }
@@ -301,8 +312,8 @@ netPropsInternal <- function(
     # unload previous dataset from RAM
     anyDM <- any.disk.matrix(data[[loadedIdx]], network[[loadedIdx]])
     vCat(verbose && anyDM, 0, "Unloading dataset from RAM...")
-    deref(dataLoaded) <- NULL
-    deref(networkLoaded) <- NULL
+    dataEnv$matrix <- NULL
+    networkEnv$matrix <- NULL
     gc()
     return(res)
   }
